@@ -238,11 +238,15 @@ nmObjGet.phiSE <- function(x, ...) {
   .obj <- x[[1]]
   .phi <- .obj$phiC
   if (is.null(.phi)) return(NULL)
-  .ret <- as.data.frame(t(vapply(seq_along(.phi), function(i) {
+  .d1 <- dim(.phi[[1]])[1]
+  .ret <- vapply(seq_along(.phi), function(i) {
     .cov <- .phi[[i]]
     suppressWarnings(sqrt(diag(.cov)))
-  }, double(dim(.phi[[1]])[1]))))
+  }, double(.d1), USE.NAMES=FALSE)
+  dim(.ret) <- c(.d1, length(.phi))
+  dimnames(.ret) <- list(colnames(.phi[[1]]), names(.phi))
   names(.ret) <- paste0("se(", names(.ret), ")")
+  .ret <- as.data.frame(t(.ret))
   .id <- seq_along(.phi)
   if (!is.null(names(.phi))) {
     .id <- names(.phi)
@@ -256,12 +260,16 @@ attr(nmObjGet.phiSE, "desc") <- "standard error of each individual's eta (if pre
 nmObjGet.phiRSE <- function(x, ...) {
   .obj <- x[[1]]
   .phi <- .obj$phiC
-  .eta <- .obj$eta[,-1]
+  .eta <- .obj$eta[,-1, drop=FALSE]
   if (is.null(.phi)) return(NULL)
-  .ret <- as.data.frame(t(vapply(seq_along(.phi), function(i) {
+  .d1 <-dim(.phi[[1]])[1]
+  .ret <-vapply(seq_along(.phi), function(i) {
     .cov <- .phi[[i]]
-    suppressWarnings(sqrt(diag(.cov))/unlist(.eta[i,])*100)
-  }, double(dim(.phi[[1]])[1]))))
+    suppressWarnings(sqrt(diag(.cov))/unlist(.eta[i,, drop=FALSE])*100)
+  }, double(.d1), USE.NAMES=FALSE)
+  dim(.ret) <- c(.d1, length(.phi))
+  dimnames(.ret) <- list(colnames(.phi[[1]]), names(.phi))
+  .ret <- as.data.frame(t(.ret))
   names(.ret) <- paste0("rse(", names(.ret), ")%")
   .id <- seq_along(.phi)
   if (!is.null(names(.phi))) {
@@ -308,9 +316,19 @@ nmObjGet.idLvl <- function(x, ...){
 .dataMergeStub <- function(obj) {
   .env      <- obj$env
   .origData <- obj$origData
-  .origData$nlmixrRowNums <- seq_along(.origData[, 1])
+  .origData$nlmixrRowNums <- seq_len(nrow(.origData))
+  # add llikObs
+  .llikObs <- FALSE
   if (exists("llikObs", obj$env)) {
+    if (length(obj$env$llikObs) == length(.origData$nlmixrRowNums)) {
       .origData$nlmixrLlikObs <- obj$env$llikObs
+      .llikObs <- TRUE
+    } else {
+      .llik0 <- data.frame(nlmixrRowNums=obj$dataSav$nlmixrRowNums, llikObs=obj$env$llikObs)
+      .llik0 <- .llik0[.llik0$nlmixrRowNums != 0,]
+      .origData <- merge(.origData, .llik0, by="nlmixrRowNums", all.x=TRUE)
+      .origData <- .origData[order(.origData$nlmixrRowNums),]
+    }
   }
   .fitData <- as.data.frame(obj)
   if (is.null(.fitData$EVID)) .fitData$EVID <- 0
